/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.key.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ObjectUtil;
import com.yiring.app.domain.key.KeyAlarmLog;
import com.yiring.app.domain.key.KeyAlarmLogRepository;
import com.yiring.app.param.key.KeyAlarmLogFindParam;
import com.yiring.app.service.key.KeyAlarmLogService;
import com.yiring.app.vo.key.KeyAlarmLogVo;
import com.yiring.auth.domain.user.User;
import com.yiring.common.core.Result;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.util.List;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 按键报警推送
 *
 * @author LJ-2204
 * @date 2022/5/13
 */

@Transactional(rollbackFor = RuntimeException.class)
@Service
@Slf4j
public class KeyAlarmLogServiceImpl implements KeyAlarmLogService {

    @Resource
    KeyAlarmLogRepository keyAlarmLogRepository;

    @Override
    public Result<PageVo<KeyAlarmLogVo>> page(KeyAlarmLogFindParam param, PageParam pageParam) {
        Specification<KeyAlarmLog> specification = (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (ObjectUtil.isNotEmpty(param.getId())) {
                predicates.add(
                    cb.equal(root.get(KeyAlarmLog.Fields.keyAlarm).get(BasicEntity.Fields.id), param.getId())
                );
            }

            if (ObjectUtil.isNotEmpty(param.getMobile())) {
                predicates.add(
                    cb.like(root.get(KeyAlarmLog.Fields.user).get(User.Fields.mobile), "%" + param.getMobile() + "%")
                );
            }

            if (ObjectUtil.isNotEmpty(param.getStatus())) {
                predicates.add(cb.equal(root.get(KeyAlarmLog.Fields.status), param.getStatus()));
            }

            if (ObjectUtil.isNotEmpty(param.getType())) {
                predicates.add(cb.equal(root.get(KeyAlarmLog.Fields.types), param.getType().toString()));
            }

            Order order = cb.desc(root.get(KeyAlarmLog.Fields.user).get(User.Fields.realName));

            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };

        Pageable pageable = PageRequest.of(pageParam.getPageNo() - 1, pageParam.getPageSize());
        Page<KeyAlarmLog> keyAlarmLogs = keyAlarmLogRepository.findAll(specification, pageable);
        List<KeyAlarmLogVo> keyAlarmLogVos = keyAlarmLogs
            .stream()
            .map(KeyAlarmLogVo::transform)
            .collect(Collectors.toList());
        PageVo<KeyAlarmLogVo> pageVo = PageVo.build(keyAlarmLogVos, keyAlarmLogs.getTotalElements());
        return Result.ok(pageVo);
    }
}
