/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.location.turnover.impl;

import cn.hutool.core.util.ObjectUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.domain.district.District;
import com.yiring.app.domain.district.DistrictRepository;
import com.yiring.app.domain.location.LocationTurnover;
import com.yiring.app.domain.location.LocationTurnoverRepository;
import com.yiring.app.excel.perstatistics.PersonnelStatisticsExportExcel;
import com.yiring.app.param.perstatistics.PersonnelStatisticsQueryParam;
import com.yiring.app.service.location.turnover.LocationTurnoverService;
import com.yiring.app.vo.perstatistics.PersonnelStatisticsDetailVo;
import com.yiring.app.vo.perstatistics.PersonnelStatisticsVo;
import com.yiring.app.vo.perstatistics.TurnoverResultVo;
import com.yiring.auth.domain.dept.Department;
import com.yiring.auth.domain.user.User;
import com.yiring.common.core.Result;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.persistence.criteria.*;
import javax.servlet.http.HttpServletResponse;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

/**
 * @author tzl
 * @version 1.0
 * @description:
 * @date 2022/5/15 11:41
 */
@Service
public class LocationTurnoverServiceImpl implements LocationTurnoverService {

    @Resource
    DistrictRepository districtRepository;

    @Resource
    LocationTurnoverRepository locationTurnoverRepository;

    @Resource
    EntityManager em;

    @Override
    public Result<PageVo<PersonnelStatisticsVo>> pageInfo(
        PersonnelStatisticsQueryParam personnelStatisticsQueryParam,
        PageParam pageParam
    ) {
        List<District> all = districtRepository.findAll();
        Map<Long, String> maps = all.stream().collect(Collectors.toMap(District::getId, District::getName));
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<TurnoverResultVo> cq = cb.createQuery(TurnoverResultVo.class);
        Root<LocationTurnover> root = cq.from(LocationTurnover.class);
        Path<Object> sourceId = root.get(LocationTurnover.Fields.sourceId);
        Path<Object> enter = root.get(LocationTurnover.Fields.enter);
        Expression<Long> num = cb.count(root.get(BasicEntity.Fields.id));

        cq.multiselect(num, sourceId, enter).groupBy(sourceId, enter);

        List<Predicate> list = new ArrayList<>();
        list.add(cb.equal(root.get(LocationTurnover.Fields.type), LocationTurnover.Type.DISTRICT));
        if (ObjectUtil.isNotNull(personnelStatisticsQueryParam.getSourceId())) {
            list.add(cb.equal(root.get(LocationTurnover.Fields.sourceId), personnelStatisticsQueryParam.getSourceId()));
        }
        if (
            ObjectUtil.isNotNull(personnelStatisticsQueryParam.getStartTime()) &&
            ObjectUtil.isNotNull(personnelStatisticsQueryParam.getEndTime())
        ) {
            list.add(
                cb.between(
                    root.get(LocationTurnover.Fields.time),
                    personnelStatisticsQueryParam.getStartTime(),
                    personnelStatisticsQueryParam.getEndTime()
                )
            );
        } else {
            list.add(
                cb.between(
                    root.get(LocationTurnover.Fields.time),
                    LocalDateTime.of(LocalDate.now(), LocalTime.MIN),
                    LocalDateTime.of(LocalDate.now(), LocalTime.MAX)
                )
            );
        }
        cq.where(list.toArray(new Predicate[0]));
        List<TurnoverResultVo> resultList = em.createQuery(cq).getResultList();
        List<Long> collect = resultList
            .stream()
            .map(TurnoverResultVo::getSourceId)
            .distinct()
            .collect(Collectors.toList());

        List<PersonnelStatisticsVo> pageList = new ArrayList<>();
        for (Long id : collect) {
            PersonnelStatisticsVo personnelStatisticsVo = new PersonnelStatisticsVo();
            personnelStatisticsVo.setSourceId(id);
            personnelStatisticsVo.setRegion(maps.get(id));
            personnelStatisticsVo.setAdmissionNumber(0L);
            personnelStatisticsVo.setAttendance(0L);
            if (
                ObjectUtil.isNotNull(personnelStatisticsQueryParam.getStartTime()) &&
                ObjectUtil.isNotNull(personnelStatisticsQueryParam.getEndTime())
            ) {
                personnelStatisticsVo.setStartTime(personnelStatisticsQueryParam.getStartTime());
                personnelStatisticsVo.setEndTime(personnelStatisticsQueryParam.getEndTime());
            } else {
                personnelStatisticsVo.setStartTime(LocalDateTime.of(LocalDate.now(), LocalTime.MIN));
                personnelStatisticsVo.setEndTime(LocalDateTime.of(LocalDate.now(), LocalTime.MAX));
            }
            for (TurnoverResultVo turnoverResultVo : resultList) {
                if (id.equals(turnoverResultVo.getSourceId())) {
                    if (turnoverResultVo.getEnter()) {
                        personnelStatisticsVo.setAdmissionNumber(turnoverResultVo.getNum());
                    } else {
                        personnelStatisticsVo.setAttendance(turnoverResultVo.getNum());
                    }
                }
            }
            pageList.add(personnelStatisticsVo);
        }
        PageVo<PersonnelStatisticsVo> build = PageVo.build(pageList, pageList.size());
        return Result.ok(build);
    }

    @Override
    public Result<PageVo<PersonnelStatisticsVo>> indexInfo(
        PersonnelStatisticsQueryParam personnelStatisticsQueryParam
    ) {
        List<District> all = districtRepository.findAll();
        Map<Long, String> maps = all.stream().collect(Collectors.toMap(District::getId, District::getName));
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<TurnoverResultVo> cq = cb.createQuery(TurnoverResultVo.class);
        Root<LocationTurnover> root = cq.from(LocationTurnover.class);
        Path<Object> sourceId = root.get(LocationTurnover.Fields.sourceId);
        Path<Object> enter = root.get(LocationTurnover.Fields.enter);
        Expression<Long> num = cb.count(root.get(BasicEntity.Fields.id));

        cq.multiselect(num, sourceId, enter).groupBy(sourceId, enter);

        List<Predicate> list = new ArrayList<>();
        list.add(cb.equal(root.get(LocationTurnover.Fields.type), LocationTurnover.Type.DISTRICT));
        if (ObjectUtil.isNotNull(personnelStatisticsQueryParam.getSourceId())) {
            list.add(cb.equal(root.get(LocationTurnover.Fields.sourceId), personnelStatisticsQueryParam.getSourceId()));
        }
        if (
            ObjectUtil.isNotNull(personnelStatisticsQueryParam.getStartTime()) &&
            ObjectUtil.isNotNull(personnelStatisticsQueryParam.getEndTime())
        ) {
            list.add(
                cb.between(
                    root.get(LocationTurnover.Fields.time),
                    personnelStatisticsQueryParam.getStartTime(),
                    personnelStatisticsQueryParam.getEndTime()
                )
            );
        }
        cq.where(list.toArray(new Predicate[0]));
        List<TurnoverResultVo> resultList = em.createQuery(cq).getResultList();
        List<Long> collect = resultList
            .stream()
            .map(TurnoverResultVo::getSourceId)
            .distinct()
            .collect(Collectors.toList());

        List<PersonnelStatisticsVo> pageList = new ArrayList<>();
        for (Long id : collect) {
            PersonnelStatisticsVo personnelStatisticsVo = new PersonnelStatisticsVo();
            personnelStatisticsVo.setSourceId(id);
            personnelStatisticsVo.setRegion(maps.get(id));
            personnelStatisticsVo.setAdmissionNumber(0L);
            personnelStatisticsVo.setAttendance(0L);
            for (TurnoverResultVo turnoverResultVo : resultList) {
                if (id.equals(turnoverResultVo.getSourceId())) {
                    if (turnoverResultVo.getEnter()) {
                        personnelStatisticsVo.setAdmissionNumber(turnoverResultVo.getNum());
                    } else {
                        personnelStatisticsVo.setAttendance(turnoverResultVo.getNum());
                    }
                }
            }
            pageList.add(personnelStatisticsVo);
        }
        PageVo<PersonnelStatisticsVo> build = PageVo.build(pageList, pageList.size());
        return Result.ok(build);
    }

    @Override
    public void export(
        PersonnelStatisticsQueryParam personnelStatisticsQueryParam,
        HttpServletResponse httpServletResponse
    ) {
        List<District> all = districtRepository.findAll();
        Map<Long, String> maps = all.stream().collect(Collectors.toMap(District::getId, District::getName));
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<TurnoverResultVo> cq = cb.createQuery(TurnoverResultVo.class);
        Root<LocationTurnover> root = cq.from(LocationTurnover.class);
        Path<Object> sourceId = root.get(LocationTurnover.Fields.sourceId);
        Path<Object> enter = root.get(LocationTurnover.Fields.enter);
        Expression<Long> num = cb.count(root.get(BasicEntity.Fields.id));

        cq.multiselect(num, sourceId, enter).groupBy(sourceId, enter);

        List<Predicate> list = new ArrayList<>();
        list.add(cb.equal(root.get(LocationTurnover.Fields.type), LocationTurnover.Type.DISTRICT));
        if (ObjectUtil.isNotNull(personnelStatisticsQueryParam.getSourceId())) {
            list.add(cb.equal(root.get(LocationTurnover.Fields.sourceId), personnelStatisticsQueryParam.getSourceId()));
        }
        if (
            ObjectUtil.isNotNull(personnelStatisticsQueryParam.getStartTime()) &&
            ObjectUtil.isNotNull(personnelStatisticsQueryParam.getEndTime())
        ) {
            list.add(
                cb.between(
                    root.get(LocationTurnover.Fields.time),
                    personnelStatisticsQueryParam.getStartTime(),
                    personnelStatisticsQueryParam.getEndTime()
                )
            );
        }
        cq.where(list.toArray(new Predicate[0]));
        List<TurnoverResultVo> resultList = em.createQuery(cq).getResultList();
        List<Long> collect = resultList
            .stream()
            .map(TurnoverResultVo::getSourceId)
            .distinct()
            .collect(Collectors.toList());

        List<PersonnelStatisticsVo> pageList = new ArrayList<>();
        for (Long id : collect) {
            PersonnelStatisticsVo personnelStatisticsVo = new PersonnelStatisticsVo();
            personnelStatisticsVo.setSourceId(id);
            personnelStatisticsVo.setRegion(maps.get(id));
            personnelStatisticsVo.setAdmissionNumber(0L);
            personnelStatisticsVo.setAttendance(0L);
            for (TurnoverResultVo turnoverResultVo : resultList) {
                if (id.equals(turnoverResultVo.getSourceId())) {
                    if (turnoverResultVo.getEnter()) {
                        personnelStatisticsVo.setAdmissionNumber(turnoverResultVo.getNum());
                    } else {
                        personnelStatisticsVo.setAttendance(turnoverResultVo.getNum());
                    }
                }
            }
            pageList.add(personnelStatisticsVo);
        }

        List<PersonnelStatisticsExportExcel> data = pageList
            .stream()
            .map(per -> {
                PersonnelStatisticsExportExcel PersonnelStatisticsExportExcel = new PersonnelStatisticsExportExcel();
                BeanUtils.copyProperties(per, PersonnelStatisticsExportExcel);
                return PersonnelStatisticsExportExcel;
            })
            .collect(Collectors.toList());

        try (
            DefaultStreamExcelBuilder<PersonnelStatisticsExportExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(PersonnelStatisticsExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(data);

            String fileName = URLEncoder.encode("人员统计信息.xlsx", StandardCharsets.UTF_8);
            httpServletResponse.setContentType("application/octet-stream");
            httpServletResponse.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            httpServletResponse.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = httpServletResponse.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            throw new RuntimeException("导出人员统计信息失败: " + e.getMessage());
        }
    }

    @Override
    public Result<PageVo<PersonnelStatisticsDetailVo>> selectInfo(
        PersonnelStatisticsQueryParam personnelStatisticsQueryParam,
        PageParam pageParam
    ) {
        Page<LocationTurnover> all = locationTurnoverRepository.findAll(
            condition(personnelStatisticsQueryParam),
            PageParam.toPageable(pageParam)
        );
        List<PersonnelStatisticsDetailVo> data = all
            .get()
            .map(locationTurnover -> {
                PersonnelStatisticsDetailVo vo = new PersonnelStatisticsDetailVo();
                BeanUtils.copyProperties(locationTurnover, vo);
                PersonnelStatisticsDetailVo.UserVo build = PersonnelStatisticsDetailVo.UserVo
                    .builder()
                    .realName(locationTurnover.getUser().getRealName())
                    .deptName(locationTurnover.getUser().getDepartment().getName())
                    .uuid(locationTurnover.getUser().getUuid())
                    .build();
                vo.setUserVo(build);
                return vo;
            })
            .collect(Collectors.toList());
        PageVo<PersonnelStatisticsDetailVo> resultVo = PageVo.build(data, all.getTotalElements());
        return Result.ok(resultVo);
    }

    public Specification<LocationTurnover> condition(PersonnelStatisticsQueryParam personnelStatisticsQueryParam) {
        return (root, query, criteriaBuilder) -> {
            Join<LocationTurnover, User> join = root.join("user", JoinType.LEFT);
            Join<User, Department> department = join.join("department", JoinType.LEFT);
            List<Predicate> list = new ArrayList<>();
            //查询条件
            if (ObjectUtil.isNotNull(personnelStatisticsQueryParam.getSourceId())) {
                list.add(
                    criteriaBuilder.equal(
                        root.get(LocationTurnover.Fields.sourceId),
                        personnelStatisticsQueryParam.getSourceId()
                    )
                );
            }

            if (
                Objects.nonNull(personnelStatisticsQueryParam.getStartTime()) &&
                Objects.nonNull(personnelStatisticsQueryParam.getEndTime())
            ) {
                list.add(
                    criteriaBuilder.between(
                        root.get(LocationTurnover.Fields.time),
                        personnelStatisticsQueryParam.getStartTime(),
                        personnelStatisticsQueryParam.getEndTime()
                    )
                );
            }

            if (ObjectUtil.isNotNull(personnelStatisticsQueryParam.getDepartmentId())) {
                list.add(
                    criteriaBuilder.equal(
                        department.get(BasicEntity.Fields.id),
                        personnelStatisticsQueryParam.getDepartmentId()
                    )
                );
            }

            Predicate[] predicates = new Predicate[list.size()];
            query.where(list.toArray(predicates));
            return criteriaBuilder.and(list.toArray(predicates));
        };
    }
}
