/* (C) 2022 YiRing, Inc. */
package com.yiring.app.vo.rehearsal;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.yiring.app.constant.rehearsal.RehearsalPlanStatusEnum;
import com.yiring.app.constant.rehearsal.RiskLevelEnum;
import com.yiring.app.domain.rehearsal.RehearsalPlan;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serial;
import java.io.Serializable;
import java.time.LocalDateTime;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/10 13:48
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder
@ApiModel("演练计划VO")
public class RehearsalPlanVo implements Serializable {

    @Serial
    private static final long serialVersionUID = 6580835031371232948L;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "id", example = "1", required = true)
    private Long id;

    @ApiModelProperty(value = "演练主题", example = "红色演练")
    private String topical;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "事故点id", example = "1522844636060585984")
    private Long accidentSpotId;

    @ApiModelProperty(value = "事故点名称", example = "事故点1")
    private String accidentSpotName;

    @ApiModelProperty(value = "周边范围(事故点周围多少米)", example = "50")
    private Integer scope;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "撤离区id", example = "1523562795864428544")
    private Long evacuationZoneId;

    @ApiModelProperty(value = "撤离点名称", example = "撤离点1")
    private String evacuationZoneName;

    @ApiModelProperty(value = "风险程度code", example = "1")
    private Integer riskLevel;

    @ApiModelProperty(value = "风险等级名称", example = "红色风险")
    private String riskLevelName;

    @ApiModelProperty(value = "事故信息", example = "气体泄露")
    private String accidentMsg;

    @ApiModelProperty(value = "演练时间", example = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime rehearsalTime;

    @ApiModelProperty(value = "演练计划的状态code", example = "1")
    private Integer status;

    @ApiModelProperty(value = "演练计划的状态名称", example = "未开始")
    private String statusName;

    public static RehearsalPlanVo transform(RehearsalPlan rehearsalPlan) {
        return RehearsalPlanVo
            .builder()
            .id(rehearsalPlan.getId())
            .topical(rehearsalPlan.getTopical())
            .accidentSpotId(rehearsalPlan.getAccidentSpot().getId())
            .accidentSpotName(rehearsalPlan.getAccidentSpot().getName())
            .scope(rehearsalPlan.getScope())
            .evacuationZoneId(rehearsalPlan.getEvacuationZone().getId())
            .evacuationZoneName(rehearsalPlan.getEvacuationZone().getName())
            .riskLevel(rehearsalPlan.getRiskLevel())
            .riskLevelName(RiskLevelEnum.getByCode(rehearsalPlan.getRiskLevel()))
            .accidentMsg(rehearsalPlan.getAccidentMsg())
            .rehearsalTime(rehearsalPlan.getRehearsalTime())
            .status(rehearsalPlan.getStatus())
            .statusName(RehearsalPlanStatusEnum.getByCode(rehearsalPlan.getStatus()))
            .build();
    }
}
