/* (C) 2022 YiRing, Inc. */
package com.yiring.app.web.visitor;

import com.yiring.app.param.visitor.VisitorParam;
import com.yiring.app.param.visitor.VisitorQueryParam;
import com.yiring.app.service.visitor.VisitorService;
import com.yiring.app.vo.visitor.VisitorDetailsVo;
import com.yiring.app.vo.visitor.VisitorVo;
import com.yiring.common.core.Result;
import com.yiring.common.param.IndexParam;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author tzl
 * 2022/4/21 9:50
 */
@Slf4j
@Validated
@SuppressWarnings({ "deprecation" })
@Api(tags = "访客", description = "Visitor")
@RestController
@RequestMapping("/visitor/")
public class VisitorController {

    @Resource
    VisitorService visitorService;

    @ApiOperation(value = "新增访客信息")
    @PostMapping("saveVisitor")
    public Result<VisitorDetailsVo> saveVisitor(@Valid VisitorParam visitorParam) {
        return visitorService.saveVisitor(visitorParam);
    }

    @ApiOperation(value = "查看访客信息详情")
    @GetMapping("selectVisitor")
    public Result<VisitorDetailsVo> selectVisitor(@Valid IndexParam indexParam) {
        return visitorService.selectVisitor(indexParam);
    }

    @ApiOperation(value = "查看访客信息(分页)")
    @GetMapping("pageVisitor")
    public Result<PageVo<VisitorVo>> pageVisitor(@Valid VisitorQueryParam visitorQueryParam) {
        return visitorService.pageVisitor(visitorQueryParam);
    }

    @ApiOperation(value = "导出访客信息", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    @PostMapping("exportVisitor")
    public void exportVisitor(HttpServletResponse response, @Valid VisitorQueryParam visitorQueryParam) {
        visitorService.exportVisitor(visitorQueryParam, response);
    }
}
